// Zapacab Client Functionality
var pingInterval = 4;
var pingCount = 1;
var lastPing = 0;

var geoWatchPositionId = null; // Continously watch geo position
var geoTimeout = 20000; // How long we try find a geoposition for
var geoMaxAge = 30000; // How stale a geoposition can be
var currentGeoPosition = null;
var currentGeoAddress = null;

var map = null;
var clientMarker = null;
var driverMarker = null;

var storeClientLocation = false;
var isSearchPage = false;
var updatePreviewInterval = null;
var clientLocation = null;
var updateCountdownInterval = null;

var clientLocationManual = false;
var gotFirstGeolocation = false;
var updateLocationTimeout = null;
var settingClientLocation = false;

var countryRestriction = "za";

// Functionality to disable dragging the map
var dragFlag = false;
var start = 0, end = 0;
function thisTouchStart(e)
{
    dragFlag = true;
    start = e.touches[0].pageY;
}

function thisTouchEnd()
{
    dragFlag = false;
}

function thisTouchMove(e)
{
    if (!dragFlag) {
        return;
    }
    end = e.touches[0].pageY;
    window.scrollBy(0, start - end);
}

var trip = new function Trip() {
    this.tripId = null;
    this.location = null;
    this.map = null;
    this.driverPosition = null;

    this.driverMarker = null;
    this.clientMarker = null;

    this.updateInterval = 5000;

    this.tripStatus = null;

    this.init = function(tripId) {
        this.tripId = tripId;

        // Define Marker properties
        this.driverMarkerImage = new google.maps.MarkerImage("/static/images/icon_taxi_available.png",
            null, null, null, new google.maps.Size(32, 44));
        this.clientMarkerImage = new google.maps.MarkerImage('/static/images/icon_client.png',
            null, null, null,new google.maps.Size(32, 44));
    };

    this.showDriverMap = function() {
        $(".map-top-actions").addClass("trip-map-actions");
        $(".map-bottom-actions").addClass("trip-map-actions");
        $(".map-bottom-actions").addClass("map-actions");

        var mapOptions = {
            center: new google.maps.LatLng(-33.9248, 18.4240),
            zoom: 16,
            mapTypeId: google.maps.MapTypeId.ROADMAP,
            mapTypeControl: false,
            streetViewControl: false,
            zoomControl: false,
            panControl: false,
            scrollwheel: false,
            overviewMapControl: false,
            draggable: false
        };

        this.map = new google.maps.Map(document.getElementById("map-canvas"), mapOptions);
        document.getElementById("map-canvas").addEventListener("touchstart", thisTouchStart, true);
        document.getElementById("map-canvas").addEventListener("touchend", thisTouchEnd, true);
        document.getElementById("map-canvas").addEventListener("touchmove", thisTouchMove, true);
        if (this.driverMarker) {
            this.map.setCenter(this.driverMarker.position);
            this.driverMarker.setMap(this.map);
        }

        $(".driver-map-holder").show();
    };

    this.hideDriverMap = function() {
        if ($(".driver-map-holder").is(':visible')) {
            $(".map-top-actions").removeClass("trip-map-actions");
            $(".map-bottom-actions").removeClass("trip-map-actions");
            $(".map-bottom-actions").removeClass("map-actions");
            $(".driver-map-holder").hide();
        }
    };

    this.fail = function() {
        $(".zc-dialog").hide();
        $("#failed-dialog").show();
        trip.updateInterval = 100000;

        if (trip.updateTimeout) {
            clearTimeout(trip.updateTimeout);
            trip.updateTimeout = null;
        }
    };

    this.redirect = function() {
        if (trip.location) {
            location.href="/location/" + trip.location;
        }
        else {
            location.href='/';
        }
    };

    this.updateTripStatus = function() {
        $.ajax({
            type: "GET",
            url: "/api/latest/c/update/" + this.tripId + "/",
            cache: false,
            dataType: "json",
            success: function(data) {
                var t = data.trip;

                if (t.status == 'accepted') {
                    if (!trip.driverPosition) {
                        $(".zc-dialog").hide();
                        $("#arriving-dialog").show();
                    }

                    if (t.support_num) {
                        $(".support_num").html("Call support " + t.support_num);
                        $(".support_num").attr('href','tel:'+ t.support_num);
                    }
                    else {
                        $(".support_num").html("Call support 0836334370");
                    }

                    trip.driverPosition = new google.maps.LatLng(
                        t.driver_position.split(';')[0],
                        t.driver_position.split(';')[1]);
                    trip.setDriverMarker(trip.driverPosition);

                    if (t.driver_number) {
                        if (t.driver_company) {
                            $("#cab-info").html(t.driver_company + " driver " + t.driver_name + " is on the way<br />Call: " +
                                "<a href='tel:" + t.driver_number + "' class='ui-link'>" + t.driver_number + "</a>");
                        }
                        else {
                            $("#cab-info").html(t.driver_name + " is on the way<br />Call: " +
                                "<a href='tel:" + t.driver_number + "' class='ui-link'>" + t.driver_number + "</a>");
                        }
                    }

                    $("#arriving-time").html(
                        parseInt(t.driver_distance / 500 + 1, 10));
                }
                else if (t.status == 'onboard') {
                    $(".zc-dialog").hide();
                    trip.hideDriverMap();
                    $("#transit-dialog").show();
                    trip.updateInterval = 10000;
                }
                else if (t.status == 'failed' || t.status == 'no drivers') {
                    if (t.status != trip.tripStatus) {
                        trip.fail();
                    }
                }
                else if (t.status == 'cancelled') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert('Your trip has been cancelled.', function() {
                            trip.redirect();
                        });
                    }
                }
                else if (t.status == 'driver cancelled') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert("Unfortunately your driver has cancelled. Please try again if you still need a cab", function() {
                            trip.redirect();
                        });
                    }
                }
                else if (t.status == 'no show') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert("Unfortunately your driver couldn't contact you and has cancelled. Please try again if you still need a cab", function() {
                            trip.redirect();
                        });
                    }
                }
                else if (t.status == 'finished') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert('Your trip has been finished. ' +
                            'We hope you had a great ride and see you again soon.', function() {
                                trip.redirect();
                        });
                    }
                }

                trip.tripStatus = t.status;

            }
        });

        this.updateTimeout = setTimeout(function() {
            trip.updateTripStatus();
        }, trip.updateInterval);
    };

    this.setMapLocation = function(latlng) {
        // Center the map on a location
        if (trip.map) {
            trip.map.setCenter(latlng);
        }
    };

    this.setClientMarker = function(pos){
        // Set a client marker on our map
        if (pos.coords) {
            pos = new google.maps.LatLng(pos.coords.latitude, pos.coords.longitude);
        }

        if (!this.clientMarker) {
            this.clientMarker = new google.maps.Marker({
                position: pos,
                map: this.map,
                icon: this.clientMarkerImage,
            });
        }

        if (positionsDifferent(this.clientMarker.position, pos)) {
            this.clientMarker.setPosition(pos);
        }
    };

    this.setDriverMarker = function(pos){
        if (pos.coords) {
            pos = new google.maps.LatLng(pos.coords.latitude, pos.coords.longitude);
        }

        if (!this.driverMarker) {
            this.driverMarker = new google.maps.Marker({
                position: pos,
                map: trip.map,
                icon: this.driverMarkerImage,
            });

            this.driverMarker.setPosition(pos);
            this.setMapLocation(pos);
        }

        if (positionsDifferent(this.driverMarker.position, pos)) {
            this.driverMarker.setPosition(pos);
            this.setMapLocation(pos);
        }

        if (!this.driverMarker.map) {
           this.driverMarker.setMap(this.map);
        }
    };

    this.cancelTrip = function() {
        bootbox.dialog({
            title: "Cancel trip?",
            message: "Are you sure you want to cancel this trip?",
            buttons: {
                no: {
                    label: "No",
                    className: "btn-default",
                    callback: function() {
                    }
                },
                yes: {
                    label: "Yes",
                    className: "btn-primary",
                    callback: function() {
                        $.ajax({
                          type: 'GET',
                          url: "/api/latest/c/cancel/" + trip.tripId + "/",
                          cache: false,
                          success: function(data) {
                            trip.tripStatus = "cancelled";
                            bootbox.alert('Your trip has been cancelled.', function() {
                                trip.redirect();
                            });
                          }
                        });
                    }
                },
            }
        });
    };

    this.startCountdownTimer = function(time, selector){
        // A countdown timer while searching for trips
        var div = $(selector);
        div.show();

        var totalTime = time;
        trip.updateCountdownTimer(totalTime, selector);

        trip.updateCountdownInterval = setInterval(function(){
            totalTime -= 1;
            if(totalTime >= 0) {
                trip.updateCountdownTimer(totalTime, selector);
            }
            else {
                if (trip.updateCountdownInterval) {
                    clearInterval(trip.updateCountdownInterval);
                }

                trip.updateCountdownInterval = null;
                if (!trip.tripStatus || trip.tripStatus == "inquiry") {
                    trip.fail();
                }
            }
        }, 1000);
    };

    this.updateCountdownTimer = function(time, selector) {
        $(selector).find(".time-left").html(time);
    };
};


// Geolocation related functions

function geoResultCallback(position) {
    // We received a new position

    hidePageLoader();

    if (storeClientLocation && !clientLocationManual) {
        setClientLocation(position.coords.latitude, position.coords.longitude);

        if (map) {
            setMapLocation(new google.maps.LatLng(
                position.coords.latitude, position.coords.longitude));
        }
    }

    gotFirstGeolocation = true;
}

function geoFailure(error){
    // We have failed to get the clients position
    console.log(error);
    gotFirstGeolocation = true;

    hidePageLoader();
    $("#location-entry").attr("placeholder", "Enter pickup location").blur();

    if (storeClientLocation) {
        $("#location-error-text").html("Sorry, we weren't able to locate your current position. Please try to enter your current address manually");
        $("#location-error").show();
    }
}

function getGeolocationOnce() {
    // Get the current geolocation
    if (clientLocationManual) {
        return;
    }

    showPageLoader("Please allow location sharing and we'll start searching for cabs in your area!");
    if (navigator.geolocation) {
        navigator.geolocation.getCurrentPosition(geoResultCallback, geoFailure, {
            enableHighAccuracy:true,
            timeout:geoTimeout,
            maximumAge:geoMaxAge
        });
    }
    else {
        if (!gotFirstGeolocation) {
            bootbox.alert('Gathering of geolocation data not supported!');
        }
        gotFirstGeolocation = true;
    }
}

function getGeolocation() {
    // Get any updates to the geolocation
    showPageLoader("Please allow location sharing and we'll start searching for cabs in your area!");

    if (navigator.geolocation) {
        geoWatchPositionId = navigator.geolocation.watchPosition(geoResultCallback, geoFailure, {
            enableHighAccuracy: true,
            timeout: geoTimeout,
            maximumAge: geoMaxAge
        });
    }
    else {
        bootbox.alert('Gathering of geolocation data not supported!');
        gotFirstGeolocation = true;
    }
}

function setClientLocation(latitude, longitude, address){
    // Geocode and set client location
    console.log("setClientLocation: " + latitude + ":" + longitude + ": " + address);

    settingClientLocation = true;

    $("#location-label").html("Request pickup from:");

    params = {
        "bounds": google.maps.LatLngBounds(google.maps.LatLng(-33, 18), google.maps.LatLng(-32, 19)),
        "language": "en",
    };

    var updateMap = true;

    if (latitude && longitude)  {
        if (clientLocation) {
            var latDiff = Math.abs(clientLocation.lat() - latitude);
            var lngDiff = Math.abs(clientLocation.lng() - longitude);

            if (latDiff < 0.01 && lngDiff < 0.01) {
                updateMap = false;
            }
        }

        clientLocation = new google.maps.LatLng(latitude, longitude);
        params["latLng"] = clientLocation;
    }
    else {
        params["address"] = address;
    }

    var nextButton;

    try {
        nextButton = Ladda.create(document.getElementById("zap-step-1"));
        nextButton.start();
    }
    catch (e) {
        console.log(e);
    }

    geocoder = new google.maps.Geocoder();
    geocoder.geocode(params, function(results, status){
        if(status == google.maps.GeocoderStatus.OK) {
            $("#location-error").hide();
            gmapsResults = results;

            $("#client-area").val(getSuburb(results[0]));
            $("#client-latitude").val(clientLocation.lat());
            $("#client-longitude").val(clientLocation.lng());
            $("#client-city").val(getCity(results[0]));

            if (landing && updateMap) {
                landing.updateClientLocation(clientLocation);
            }

            updateCityInformationDisplay($("#client-city").val());

            $(".client-location-entry").attr("placeholder", "Please enter a location");

            if (!address) {
                $(".client-location-entry").val(getPrettyAddress(results[0]));
                $(".client-location").html(getPrettyAddress(results[0]));
                if (!clientLocationManual) {
                    currentGeoPosition = params["latLng"];
                    currentGeoAddress = getPrettyAddress(results[0]);
                }
            }
            else {
                $(".client-location-entry").val(address);
                $(".client-location").html(address);
            }

            $(".location-enable").removeAttr("disabled");
            startUpdatePreview();

            if (nextButton) {
                nextButton.stop();
            }
        }
        else {
            if (address) {
                $("#location-error-text").html("Sorry, we aren't able to understand this location");
                $("#location-error").show();
            }
        }

        settingClientLocation = false;
    });
}

function updateCityInformationDisplay(cityName) {
    if ($("#city-display")) {
        $.ajax({
            type: "get",
            url: "/api/v2/c/city/" + cityName + "/",
            cache: false,
            dataType: "json",
            success: function(data) {
                console.log(data);
                var display = false;
                var city = data.city;

                console.log(city);

                if (city) {
                    if (city.maximum_price) {
                        $(".city-maximum-price").html(city.maximum_price);
                        display = true;
                    }
                    if (city.minimum_fare) {
                        $(".city-minimum-fare").html(city.minimum_fare + " minimum");
                        display = true;
                    }
                }


                if (display) {
                    $("#city-display").show();
                }
                else {
                    $("#city-display").hide();
                }
            }
        });
    }
}

function startUpdatePreview() {
    // An interval to preview the number of cabs around the client's area
    if (updatePreviewInterval) {
        clearInterval(updatePreviewInterval);
        updatePreviewInterval = null;
    }

    updatePreview();
    updatePreviewInterval = setInterval(updatePreview, 30000);
}

function updatePreview() {
    // Update the preview of cabs around clients location
    if (clientLocation !== null) {
        $("#available-cabs-preview").show();
    }
    else {
        $("#available-cabs-preview").hide();
    }

    data = {
        latitude: clientLocation.lat(),
        longitude: clientLocation.lng()
    };

    $.ajax({
        data: data,
        type: "get",
        url: "/c/preview/",
        cache: false,
        success: function(data) {
            $("#available-cabs-preview").html(data);
        }
    });
}

function getClientLocation() {
    $("#id_search").button('disable');
    storeClientLocation = true;

    handleManualLocation();

    getGeolocation();
}

function handleManualLocation() {
    var autocompleteOptions = {
        componentRestrictions: {country: countryRestriction}
    };

    var addressField = document.getElementById("location-entry");
    addressAutocomplete = new google.maps.places.Autocomplete(
      addressField, autocompleteOptions);

    google.maps.event.addListener(addressAutocomplete, 'place_changed', function() {
        var place = addressAutocomplete.getPlace();

        if(typeof place.address_components == 'undefined') {
            autocompleteService = new google.maps.places.AutocompleteService();
            autocompleteService.getPlacePredictions(
                {
                    'input': place.name,
                    'offset': place.name.length,
                    'componentRestrictions': {'country': countryRestriction},
                },

                function listentoresult(list, status) {
                    if(list === null || list.length === 0) {
                        console.log("No results");
                    } else {
                        placesService = new google.maps.places.PlacesService(document.getElementById('location-entry'));
                        placesService.getDetails(
                            {'reference': list[0].reference},
                            function detailsresult(detailsResult, placesServiceStatus) {
                                $("#location-entry").val(getPrettyAddress(detailsResult));
                                console.log("detailsresult");

                                place = detailsResult;
                                if (place && place.geometry) {
                                    var location = place.geometry.location;
                                    clientLocationManual = true;
                                    setClientLocation(location.lat(), location.lng(), $("#location-entry").val());
                                }
                            }
                        );
                    }
                }
            );
        }

        console.log("place_changed");

        if (place && place.geometry) {
            var location = place.geometry.location;
            clientLocationManual = true;
            setClientLocation(location.lat(), location.lng(), $("#location-entry").val());
        }
    });

    $("#location-entry").keyup(function() {
        clientLocationManual = true;
    });

    $("#location-entry").change(function() {
        clientLocationManual = true;

        if ($("#location-entry").val() === "") {
            $("location-enable").attr("disbaled", "disabled");
        }
    });

    $("#location-entry").blur(function() {
        setTimeout(function() {
            if ($("#location-entry").val() !== "") {
                if ($("#location-entry").val() != $("#client-address").val() && !settingClientLocation) {
                    var item = $(".pac-container .pac-item:first");
                    var address = item.find(".pac-item-query").text() + ", " + item.find("span:last").text();
                    //$("#location-entry").val(address);
                    setClientLocation(null, null, address);
                }
            }
        }, 50);
    });
}
