var dispatch = new function() {
    this.map = null;
    this.markers = {};

    // Define Marker properties
    this.greenMarker = new google.maps.MarkerImage("/static/images/icon_taxi_green.png",
        null, null, null, new google.maps.Size(32, 44));
    this.orangeMarker = new google.maps.MarkerImage("/static/images/icon_taxi_available.png",
        null, null, null, new google.maps.Size(32, 44));
    this.redMarker = new google.maps.MarkerImage("/static/images/icon_taxi_busy.png",
        null, null, null, new google.maps.Size(32, 44));
    this.grayMarker = new google.maps.MarkerImage("/static/images/icon_taxi_offline.png",
        null, null, null, new google.maps.Size(32, 44));
    this.userMarker = new google.maps.MarkerImage('/static/images/icon_client.png',
        null, null, null,new google.maps.Size(32, 44));

    this.initializeMap = function() {
      var mapOptions = {
        center: new google.maps.LatLng(-33.9248, 18.4240),
        zoom: 14,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
        mapTypeControl: false,
        streetViewControl: false,
      };

      this.map = new google.maps.Map(document.getElementById("map-canvas"), mapOptions);
    };

    this.update = function() {
        $.ajax({
            type: "GET",
            dataType: "json",
            url: "/admin/drivers_map_data",
            success: function(data) {
                console.log("Fetched map data");
                drivers = data.drivers;

                for (var id in dispatch.markers) {
                    if (drivers[id] === undefined) {
                        // This driver is no longer checking in
                        dispatch.markers[id].setMap(null);
                        delete dispatch.markers[id];
                    }
                }

                for (var i in drivers) {
                    var driver = drivers[i];
                    var position = new google.maps.LatLng(driver.latitude,
                        driver.longitude);

                    var icon = dispatch.grayMarker;
                    if (driver.status == "available") {
                        icon = dispatch.greenMarker;
                    }
                    else if (driver.status == "in use") {
                        icon = dispatch.orangeMarker;
                    }
                    else if (driver.status == "busy") {
                        icon = dispatch.redMarker;
                    }

                    if (dispatch.markers[driver.id] === undefined) {
                        var marker = new google.maps.Marker({
                            position: position,
                            map: dispatch.map,
                            icon: icon,
                            title: driver.name + ": " + driver.cell_number + " - " + driver.status
                        });

                        dispatch.markers[driver.id] = marker;
                    }
                    else {
                        if (dispatch.markers[driver.id].icon != icon) {
                            dispatch.markers[driver.id].setIcon(icon);
                        }
                        dispatch.markers[driver.id].setTitle(driver.name + ": " + driver.cell_number + " - " + driver.status);
                        dispatch.markers[driver.id].setPosition(position);
                    }
                }
            }
        });
    };
};