
var STATE_INITIALIZING = 0;
var STATE_GETTING_LOCATION = 1;
var STATE_GETTING_ADDRESS = 2;
var STATE_READY = 3;
var STATE_MANUAL_ADDRESS_ENTER = 4;
var STATE_CONFIRM_PICKUP = 5;
var STATE_MONITORING_TRIP = 6;
var STATE_TRIP_FAILED = 7;
var STATE_TRIP_DONE = 8;

var STATE_GETTING_LOCATION_ERROR = 9;

var ERROR_NO_CONNECTION = 0;
var ERROR_NO_LOCATION = 1;
var ERROR_NO_CABS = 2;

var TRIP_STATUS_INQUIRY = "inquiry";
var TRIP_STATUS_ACCEPTED = "accepted";
var TRIP_STATUS_ONBOARD = "onboard";
var TRIP_STATUS_FINISHED = "finished";
var TRIP_STATUS_FAILED = "failed";
var TRIP_STATUS_NO_DRIVERS = "no drivers";
var TRIP_STATUS_CANCELLED = "cancelled";
var TRIP_STATUS_DRIVER_CANCELLED = "driver cancelled";
var TRIP_STATUS_NO_SHOW = "no show";

//var API_URL = "http://zapacab-dev.appspot.com";
var API_URL = "http://zapacab-live.appspot.com";
var MAPS_TC = "http://www.google.com/intl/en-US_US/help/terms_maps.html";
var MAX_CACHED_ADDRESSES = 5;
var INITIAL_MAP_ZOOM = 4;
var READY_ZOOM = 17;
var TRACK_ZOOM = 13;

var GET_AVAILABLE_DRIVERS_RETRY_COUNT = 5;

function validPhoneNumber(number){
    if (number.trim() == ""){
        return false;
    }

    return true;
}


$(function(){

$.fn.doubleTap = function(doubleTapCallback) {
    return this.each(function(){
        var elm = this;
        var lastTap = 0;
        $(elm).bind('touchend', function (e) {
            var now = (new Date()).valueOf();
            var diff = (now - lastTap);
            lastTap = now ;
            if (diff < 250) {
                if($.isFunction( doubleTapCallback )){
                    doubleTapCallback.call(elm, e.originalEvent);
                }
                lastTap = 0;  
            } 
                   
        });
    });
}



var Trip = Backbone.Model.extend({
    idAttribute: "uuid",
    urlRoot: API_URL + "/api/latest/c/update",
    defaults: {
        driver_company: null,
        driver_distance: null,
        driver_license_plate: null,
        driver_name: null,
        driver_number: null,
        driver_position: null,
        pickup_position:null,
        started: null,
        status: null,
        support_num: null,
        uuid: null

    },
    initialize: function(){
         _.bindAll(this, "update");
        var tripid = window.localStorage.getItem("tripid");
        if (tripid){
            this.set("uuid", tripid);
            this.fetch();
        }
        this.update_timeout = null;
        //this.on("change:uuid", "fetch")
    },
    parse: function (data) {
        return data.trip;
    },
    update: function(){
        var my_state = this.get("status");
        if (my_state === null || my_state == TRIP_STATUS_INQUIRY || my_state == TRIP_STATUS_ACCEPTED || my_state == TRIP_STATUS_ONBOARD){
            this.update_timeout = window.setTimeout(function() {
                view.model.get("trip").fetch();
                view.model.get("trip").update();
            }, 1000);
        }
    }
});

var AppModel = Backbone.Model.extend({
    defaults: {
        state: STATE_INITIALIZING,
        trip: null,
        apiurl: API_URL,
        available_drivers: null,
        pickup_location: [-27.747455, 25.4236375],
        gps_location: null,
        client_address: null,
        area: null,
        client_area: null,
        client_city: null,
        client_phone_number: null,
        getAvailableDriversInterval: null,
        getAvailableDriversRetries: 0

    },
    initialize: function(){
        _.bindAll(this, 'onDeviceReady', 'onHighAccuracyLocationSuccess', 'onHighAccuracyLocationError', 
                  'onLocationSuccess', 'onLocationError', 'getAvailableDrivers', 'tripStatusChanged',
                  'cancelTrip', 'gotoReady', 'goBack', 'storeNewAddress', 'getRecentAddresses',
                  'pickupLocationChanged', 'getLocation', 'getAddress', 'stopGetAvailableDriversInterval',
                  'startGetAvailableDriversInterval', 'startNewTrip', 'appResumed');
        this.on("change:state", this.stateChanged);
        this.on("change:pickup_location", this.pickupLocationChanged);
        var tripobj = new Trip();
        this.set("trip", tripobj)
        this.set("client_phone_number", window.localStorage.getItem("phone_number"));
        tripobj.on("change:status", this.tripStatusChanged);
        document.addEventListener('deviceready', this.onDeviceReady, false);
        var tripid = window.localStorage.getItem("trip_id");
        if (tripid && tripid !== "null"){
            this.set("pickup_location", JSON.parse(window.localStorage.getItem("pickup_location")));
            window.clearTimeout(tripobj.update_timeout); 
            tripobj.set("uuid", tripid);
            this.set("state", STATE_MONITORING_TRIP);
            tripobj.fetch();
            tripobj.update();
        }
        this.addressTimeout = null;
        //this.onDeviceReady();
    },
    reloadApp: function(){
        window.location = './index.html';
    },
    exitApp: function(){
        if(navigator.app){
            navigator.app.exitApp();
        }else if(navigator.device){
            navigator.device.exitApp();
        }
    },
    gotoReady: function(){
        this.set("state", STATE_INITIALIZING);
        this.set("available_drivers", []);
        this.getLocation();
    },
    goBack: function(){
        var state = this.get("state");
        if (state == STATE_MANUAL_ADDRESS_ENTER){
            this.set("state", STATE_READY);
        }else if (state == STATE_CONFIRM_PICKUP){
            this.set("state", STATE_READY);
        }else if ((state == STATE_READY || state == STATE_GETTING_ADDRESS || state == STATE_GETTING_LOCATION) && device.platform == "Android") {
            this.exitApp();
        }
    },  
    onDeviceReady: function(){
        if (navigator.connection.type != Connection.NONE){
            view.renderMap();
            document.addEventListener("backbutton", function(){view.backButtonClicked()}, false);
            document.addEventListener("resume", view.model.appResumed, false);
            this.getLocation();
            view.renderForPlatform(device.platform);
        }else{
            view.noConnection(this.reloadApp);
        }
    },
    appResumed: function() {
        if (this.get("state") == STATE_READY){
            this.getLocation();
        }
    },
    getLocation: function(){
        if (this.get("state") == STATE_INITIALIZING || this.get("state") == STATE_READY){
            this.set("state", STATE_GETTING_LOCATION);
        }
        if (this.get("state") != STATE_MONITORING_TRIP){
            navigator.geolocation.getCurrentPosition(this.onHighAccuracyLocationSuccess, this.onHighAccuracyLocationError, {enableHighAccuracy: true, timeout: 20000});
        }
    },
    onHighAccuracyLocationSuccess: function(position){
        this.onLocationSuccess(position);
    },
    onHighAccuracyLocationError: function(){
        console.log("onHighAccuracyLocationError");
        navigator.geolocation.getCurrentPosition(this.onLocationSuccess, this.onLocationError, {});
    },
    onLocationSuccess: function(position){
        console.log('onLocationSuccess');
        var pos = [parseFloat(position.coords.latitude), parseFloat(position.coords.longitude)];

        //workaround for zap again issue with pin being just off of location
        google.maps.event.trigger(view.map, 'resize');
        
        this.set("pickup_location", pos);
        this.set("gps_location", pos);
        this.trigger("change:gps_location");
        if (this.get("state") == STATE_GETTING_LOCATION){
            this.set("state", STATE_GETTING_ADDRESS);
        }
    },
    onLocationError: function(){
        this.set("state", STATE_GETTING_LOCATION_ERROR);
        return;
    },
    tripStatusChanged: function(){
        var tripstatus = this.get("trip").get("status");
        if (tripstatus == TRIP_STATUS_FAILED || tripstatus == TRIP_STATUS_CANCELLED || tripstatus == TRIP_STATUS_DRIVER_CANCELLED || tripstatus == TRIP_STATUS_NO_DRIVERS || tripstatus == TRIP_STATUS_NO_SHOW){
            this.set("state", STATE_TRIP_FAILED);
            window.localStorage.removeItem("trip_id");
        }else if (tripstatus == TRIP_STATUS_FINISHED){
            window.localStorage.removeItem("trip_id");
            this.set("state", STATE_TRIP_DONE);
        }
    },
    startGetAvailableDriversInterval: function(){
        this.getAvailableDrivers();
        if (this.get('getAvailableDriversInterval') === null){
            var timeout = window.setInterval(function() {view.model.getAvailableDrivers();}, 5000);
            this.set("getAvailableDriversInterval", timeout)
        }
    },
    stopGetAvailableDriversInterval: function(){
        window.clearInterval(this.get('getAvailableDriversInterval'));
        this.set("getAvailableDriversInterval", null);
    },
    stateChanged: function(){
        if (this.get("state") == STATE_READY){
            this.startGetAvailableDriversInterval();

        }else if (this.get("state") == STATE_GETTING_ADDRESS){
            this.getAddress();
        }
        if (this.get("state") != STATE_READY && this.get('getAvailableDriversInterval') !== null){
            this.stopGetAvailableDriversInterval();
        }
    },
    getCityFares: function(){
        var city = this.get("client_city");
        var display = 0;
        $.ajax({
            type: "get",
            url: this.get("apiurl") + "/api/v2/c/city/" + city + "/",
            cache: false,
            dataType: "json",
            success: function(data){
                var city = data['city'];

                if (city === null){
                    $(".city-fares-info").hide();
                    return;
                }

                if (city["maximum_price"] != "RNone" && city["minimum_price"] != "RNone"){
                    $("#city-price").html(city["minimum_price"] + "/km - " + city["maximum_price"] + "/km");
                    display = 1;
                }
                else if (city["maximum_price"] != "RNone"){
                    $("#city-price").html(city["maximum_price"] + "/km or less!");
                    display = 1;
                }
                else{
                    $("#city-price").html("Minimum of " + city["minimum_price"] + "/km");
                    display = 1;
                }
                $("#city-min-fare").html("(" + city["minimum_fare"] + " minimum)");

                if (display == 1){
                    $(".city-fares-info").show();    
                }
            },
            error: function(jqXHR, textStatus, error){

            }
        });
    },
    getAvailableDrivers: function(){
        var pos = this.get("pickup_location");
        $.ajax({url:this.get("apiurl") + "/api/latest/c/drivers/",
                timeout: 5000,
                type:"get",
                data: {"latitude": pos[0],"longitude": pos[1]},
                success: function(data) {
                    view.model.getAvailableDriversRetries = 0;
                    var drivers = [];
                    for (var d in data["drivers"]){
                        if (data["drivers"][d]["status"] == "available"){
                            drivers.push(data["drivers"][d]);
                        }
                    }
                    view.model.set("available_drivers", drivers);
                },
                error: function(jqXHR, textStatus, error){
                    if (view.model.get("getAvailableDriversRetries") >= GET_AVAILABLE_DRIVERS_RETRY_COUNT){
                        view.model.set("getAvailableDriversRetries", 0)
                        view.model.stopGetAvailableDriversInterval();
                        view.noConnection(view.model.startGetAvailableDriversInterval);
                        view.model.set("available_drivers", []);
                    }else{
                        view.model.set("getAvailableDriversRetries", view.model.get("getAvailableDriversRetries") + 1);
                    }
                    
            }
        });
    },
    storeNewAddress: function(){
        var addressobj = window.localStorage.getItem("stored_addresses");
        if (!addressobj){
            addressobj = [];
        }else{
            addressobj = JSON.parse(addressobj);
        }
        var cached = false;
        for (var x in addressobj){
            if (addressobj[x]["address"] == this.get("client_address")){
                cached = true;
                break;
            }
        }
        if (cached != true){
            addressobj = addressobj.splice(-1*MAX_CACHED_ADDRESSES, MAX_CACHED_ADDRESSES);

            var new_address = {gps: this.get("pickup_location"),
                          address: this.get("client_address")}
            addressobj.unshift(new_address);
            window.localStorage.setItem("stored_addresses", JSON.stringify(addressobj)); 
        }
    },
    getRecentAddresses: function(){
        var addressobj = window.localStorage.getItem("stored_addresses");
        if (!addressobj){
            addressobj = [];
        }else{
            addressobj = JSON.parse(addressobj);
        }
        return addressobj;
    },
    startNewTrip: function(){
        window.localStorage.setItem("phone_number", this.get("client_phone_number"));
        window.localStorage.setItem("pickup_location", JSON.stringify(this.get("pickup_location")));
        if (this.get("pickup_location") === null || this.get("client_address") === null){
            return;
        }
        this.storeNewAddress();
        var pos = this.get("pickup_location");
        var data = {
            phone_number: this.get("client_phone_number"),
            address: this.get("client_address"),
            latitude: pos[0],
            longitude: pos[1],
            city: this.get("client_city"),
            area: this.get("client_area"),
            passengers: 1,
            trip_source: "phonegap: " + device.platform
        };
        console.log("called function search", data);
        var jqxhr = $.ajax({
            url: this.get("apiurl") + "/api/latest/c/search/",
            data: data,
            type: "POST",
            error: function(jqXHR, textStatus, error){
                view.noConnection(view.model.startNewTrip);
            },
            success: function(response){
                var tripobj = view.model.get("trip");
                window.clearTimeout(tripobj.update_timeout); 
                tripobj.set("uuid", response.trip);
                window.localStorage.setItem("trip_id", response.trip);
                view.model.set("state", STATE_MONITORING_TRIP);
                tripobj.fetch();
                tripobj.update();
            }
        });
        return true; //make sure Connection Error popup hides
    },
    cancelTrip: function(){
        $.ajax({type: 'GET',
                url: this.get("apiurl") + "/api/latest/c/cancel/" + window.localStorage.getItem("trip_id") + "/",
                cache: false,
                success: function(data) {
                    //don't wait for status to reflect on update
                    view.model.get("trip").set("status", TRIP_STATUS_CANCELLED);
                },
                error: function(jqXHR, textStatus, error){
                    view.noConnection(view.model.cancelTrip);
                }
        });
    },
    pickupLocationChanged: function(){
        if (this.addressTimeout !== null){
            window.clearTimeout(this.addressTimeout);
            this.addressTimeout = null;
        }
        var appstate = this.get("state");
        if (appstate == STATE_READY || appstate == STATE_GETTING_LOCATION || appstate == STATE_GETTING_ADDRESS){
            this.addressTimeout = window.setTimeout(function() {
                view.model.set("state", STATE_GETTING_ADDRESS);
            }, 1000);
        }
    },
    getAddress: function(){
        var pos = this.get("pickup_location");
        geocoder = new google.maps.Geocoder();
        var latlng = new google.maps.LatLng(pos[0], pos[1]);

        geocoder.geocode({'latLng':latlng},
            function(results, status){
                if (status == google.maps.GeocoderStatus.OK) {
                    if (results[0]) {
                        var c_address = null;
                        var c_area = null;
                        var c_city = null;
                        var components = results[0].address_components;
                        var address = results[0].formatted_address;
                        console.log("address result from rev geo: " + address);
                        c_address = address;

                        if (components.length > 2){
                            c_area = components[2].long_name;
                        }else{
                            for (var i = 0; i < components.length; i++) {
                                var t = components[i].types;
                                if (t.indexOf("sublocality") != -1){
                                    c_area = components[i].long_name
                                }
                            }
                        }

                        for (var i = 0; i < components.length; i++){
                            if (components[i].types.length){
                                var t = components[i].types;
                                if (t.indexOf("administrative_area_level_3") != -1) {
                                    c_city = components[i].long_name
                                }
                            }
                        }
                        console.log("address:" + c_address);
                        view.model.set("client_address", c_address);
                        view.model.set("client_area", c_area);
                        view.model.set("client_city", c_city);
                        view.model.getCityFares();

                        var state = view.model.get("state");
                        if (state == STATE_GETTING_ADDRESS){
                            $('#pickup_button').attr('disabled', false);
                            view.model.set("state", STATE_READY);
                        }
                    }
                    else{
                        view.showInfoPopup("No address", "No address available for this location.")
                        console.log("no results found for geocode");
                        var state = view.model.get("state");
                        if (state == STATE_GETTING_ADDRESS){
                            view.model.set("state", STATE_READY);
                        }
                    }
                }
                else{
                    view.showInfoPopup("Error", "An error occurred while trying to get your address.")
                    var state = view.model.get("state");
                    if (state == STATE_GETTING_ADDRESS){
                        view.model.set("state", STATE_READY);
                    }
                    console.log("geocoding failed with status: " + status);
                }                
            }
        );
    },
});


var AppView = Backbone.View.extend({
    el: $('body'),
    events: {
        "touchend #address_display": "eventManualAddressEnter",
        "touchend #address_display_button": "eventManualAddressEnter",
        "click #address_search_button": "clearAddressAutocomplete", //touchend doesn't wait until button is de-focussed
        "touchend #pickup_button": "eventPickUp",
        "touchend #zap_button": "newTrip",
        "change #id_phone_number": "updatePhoneNumber",
        "touchend .backbutton": "backButtonClicked",
        "touchend .hamburger_menu": "showHamburgerPage",
        "touchend .list-group .list-group-item": "cachedLocationChosen",
        "touchend #zap_another_button": "gotoReady",
        "touchend #address_confirm": "backButtonClicked",
        "touchend .link_button": "openInBrowser",
        "touchend .menu-item": "openInBrowser",
        "touchend .close_over": "closeHamburgerPage",
    },
    initialize: function(){
        _.bindAll(this, 'render', 'mapCenterChanged', 'updateClientMarker', 'pickupLocationChanged',
                  'updateAvailableDrivers', 'eventManualAddressEnter', 'startTripCountdown', 'noConnection',
                  'updateProgressbar','tripStatusUpdated', 'updateDriverMarker', 'renderMap',
                  'appStateChanged', 'newTrip', 'cancelTrip', 'updatePhoneNumber',
                  'goToCurrentLocation', 'gpsCoordsReceived', 'updateDriverStatusbar', 'backButtonClicked',
                  'locationAutocompleteChosen', 'cachedLocationChosen', 'gotoReady', 'clearAddressAutocomplete',
                  'openInBrowser', 'showHamburgerPage', 'closeHamburgerPage', 'eventPickUp', 'renderForPlatform',
                  'updateDriverDetails', 'mapDragged'); // remember: every function that uses 'this' as the current object should be in here
        
        //shortcuts to elements
        this.statusbar = $('#statusbar_text');
        this.addressbox = $("#address_display");
        this.phone_number_input = $("#id_phone_number");

        this.window_height = $(window).height();

        this.googregex = new RegExp("http://maps.google.com/");

        if (this.model.get("client_phone_number") !== null){
            this.phone_number_input.val(this.model.get("client_phone_number"));
        }

        this.model.get("trip").on('change:status', this.tripStatusUpdated);
        this.model.get("trip").on('change:driver_position', this.updateDriverMarker);
        this.model.get("trip").on('change:driver_company', this.updateDriverDetails);
        this.model.on('change:pickup_location', this.pickupLocationChanged);
        this.model.on('change:gps_location', this.gpsCoordsReceived);
        this.model.on("change:state", this.appStateChanged);
        this.model.on("change:available_drivers", this.updateAvailableDrivers);
        this.model.on('change:client_address', function(){view.addressbox.val(view.model.get("client_address").split(",")[0]);});

        this.addressTimeout = null;
        this.updateCountdownInterval = null;
        this.clientMarker = null;
        this.driverMarker = null;
        $(".zapa_page").hide();
        $("#map_page").show();
        $("#address_display").val("");
        $('#pickup_button').attr('disabled', true);

        $("#id_phone_number").on("touchend", function(){
            $("#id_phone_number").focus();
        });
        window.onresize = function(){
            if ($(window).height() != view.window_height){
                $("#zap_button").hide();
                $(".city-fares-info").hide();
            }else{
                $("#zap_button").show();
                $(".city-fares-info").show();
            }
        }

        $("#id_phone_number").keydown(function (e) {
              if (e.which == 13) {
                $("#id_phone_number").blur();
              }
        });
    },
    noConnection: function(retry_callback){
            var exit_caption = null;
            if (device.platform == "Android"){
                exit_caption = "Exit"
            }
            this.showPopup("Internet connection required.", "Zapacab requires an active internet connection. Please make sure you are connected to the internet.", "Retry", exit_caption, retry_callback, this.model.exitApp);
    },
    renderMap: function(){
        var  pos = this.model.get("pickup_location");
        var mapOptions = {
            center: new google.maps.LatLng(pos[0], pos[1]),
            zoom: INITIAL_MAP_ZOOM,
            mapTypeId: google.maps.MapTypeId.ROADMAP,
            streetViewControl: false,
            mapTypeControl: false,
            zoomControl: false,
            zoomControlOptions: {
                position: google.maps.ControlPosition.LEFT_CENTER
            },
            disableDoubleClickZoom: true,
            styles: [
                {
                    featureType: "poi",
                    elementType: "labels",
                    stylers: [{
                        visibility: "off"
                    }]
                }
            ]
        };

        $('#over_map').css('top', '50%').css('top', '-=44px');
        $('#over_map').css('left', '50%').css('left', '-=16px');

        this.map = new google.maps.Map(document.getElementById("map-canvas"),mapOptions);
        this.map_bounds_set = 0;
        this.clientMarkerImage = new google.maps.MarkerImage("img/icon_client.png", null, null, null, new google.maps.Size(32, 44));
        this.driverMarkerImage = new google.maps.MarkerImage("img/icon_taxi_available.png", null, null, null, new google.maps.Size(32, 44));
        this.map_center_changed_listener = google.maps.event.addListener(this.map, 'center_changed', this.mapCenterChanged);
        this.map_dragged_listener = google.maps.event.addListener(this.map, 'drag', this.mapDragged);
        this.driverMarker = null;
        this.driver_markers = [];
        var addressField = document.getElementById("address_search");
        var autocompleteOptions = {componentRestrictions: {country: "za"}};
        this.addressAutocomplete = new google.maps.places.Autocomplete(addressField, autocompleteOptions);
        google.maps.event.addListener(this.addressAutocomplete, 'place_changed', this.locationAutocompleteChosen);
        this.render();

        /* TODO
        // Bias the autocomplete object to the user's geographical location,
// as supplied by the browser's 'navigator.geolocation' object.
function geolocate() {
  if (navigator.geolocation) {
    navigator.geolocation.getCurrentPosition(function(position) {
      var geolocation = new google.maps.LatLng(
          position.coords.latitude, position.coords.longitude);
      autocomplete.setBounds(new google.maps.LatLngBounds(geolocation,
          geolocation));
    });
  }
        */

        setTimeout(function(){
            $("a").on("click", function(e){
                var link_url = $(e.target).attr("href");

                //If the google image on map is clicked..
                if (link_url === undefined && e.target.parentNode.parentNode !== null){
                    link_url = $(e.target.parentNode.parentNode).attr("href");
                }

                if (link_url == MAPS_TC || view.googregex.test(link_url)){
                    window.open(link_url, '_system');
                    return false;
                }
            })
        }, 2000);

        $("#map-canvas").doubleTap(function(e){
            if (view.model.get("state") == STATE_READY){
                view.map.setZoom(view.map.getZoom() + 1);
            }   
        });

    },
    renderForPlatform: function(platform){
        if (platform == "Android"){
            $(".navbar_button").css("margin-top", "10px");
            $(".logo").css("top", "12px");
        }
    },
    gotoReady: function(){
        this.model.gotoReady();
    },
    locationAutocompleteChosen: function(){
        window.setTimeout(function(){
            var place = view.addressAutocomplete.getPlace();

            //typed enter instead of selecting, so search again and select first item.
            if(typeof place.address_components == 'undefined') {
                autocompleteService = new google.maps.places.AutocompleteService();
                autocompleteService.getPlacePredictions({
                    'input': place.name,
                    'offset': place.name.length,
                    'componentRestrictions': {"country":"za"},
                },
                function listenForResult(list, status){
                    if(list == null || list.length == 0) {
                        console.log("no place found for autocomplete");
                    }
                    else {
                        placesService = new google.maps.places.PlacesService(document.getElementById('placesAttribution'));
                        placesService.getDetails(
                            {'reference': list[0].reference},
                            function detailsresult(detailsResult, placesServiceStatus) {
                                var place = detailsResult;
                                console.log(place.id);
                                console.log(place.formatted_address);
                                var location = [place.geometry.location.lat(), place.geometry.location.lng()];
                                view.model.set("state", STATE_READY);
                                view.model.set("pickup_location", location);
                            }
                        );
                    }
                });
            }
            else {
                console.log(place.id);
                console.log(place.formatted_address);
                var location = [place.geometry.location.lat(), place.geometry.location.lng()];
                view.model.set("state", STATE_READY);
                view.model.set("pickup_location", location);
            }
        }, 500)
    },
    clearAddressAutocomplete: function(){
        $("#address_search").val("");
        $("#address_search").focus();
    },
    cachedLocationChosen: function(e){
        $("#address_search").blur();
        window.setTimeout(function(){
            var cached_address = view.model.getRecentAddresses()[$(e.target).data("index")];
            view.model.set("state", STATE_READY);
            view.model.set("pickup_location", cached_address.gps);

        }, 500)
        
    },
    openInBrowser: function(e){
        window.open($(e.target).data("link"), '_system');
    },
    backButtonClicked: function(){
        if ($("#hamburger_page").is(":visible")){
            this.closeHamburgerPage();
        }else{
            $(".form-control").blur();
            window.setTimeout(function(){
                view.model.goBack();
            }, 100);
        }

    },
    gpsCoordsReceived: function(){
        this.map.setZoom(READY_ZOOM);
    },
    updatePhoneNumber: function(){
        this.model.set("client_phone_number", this.phone_number_input.val());
    },
    showHamburgerPage: function(){
        $("#hamburger_page").show();
    },
    closeHamburgerPage: function(){
        $("#hamburger_page").hide();
    },
    setNumberWidth: function() {
            var new_width = document.getElementById('address_confirm').clientWidth;
            //$("#id_phone_number").width(new_width - 10);
    },
    render: function(){
        var state = this.model.get("state");
        if (state == STATE_MONITORING_TRIP){
            if (this.map.getZoom() == INITIAL_MAP_ZOOM){
                //app restarted to this state;
                this.map.setZoom(READY_ZOOM);
            }
            this.map.setOptions({draggable: false});
            $(".current-location").hide();
        }else{
            this.map.setOptions({draggable: true});
            $(".current-location").show();
            $(".hamburger_menu").show();
            $("#cancel_button").hide();
        }
        if (state == STATE_READY || state == STATE_INITIALIZING){
            $("#over_map").show();
            $(".zapa_page").hide();
            $("#map_page").show();
            google.maps.event.trigger(this.map, 'resize');
            $("#mapwrapper").css("top", "0");
            $("#address_action").show();
            $("#pickup_button_container").show();
            $("#cancel_zap_container").hide();
            $(".pcontainer").hide();
            this.hideClientMarker();
            $(".current-location").show();
            $("#address_search").blur();
        }else if (state == STATE_MANUAL_ADDRESS_ENTER){

            $(".list-group").empty();
            var recent_addresses = this.model.getRecentAddresses();
            for (var x in recent_addresses){
                $(".list-group").append('<a href="#" class="list-group-item" data-index="' + x +'">' + recent_addresses[x].address.split(",")[0] + '</a>');
            }
            if (recent_addresses.length < 1){
                $("#recent_title").hide();
            }else{
                $("#recent_title").show();
            }

            $(".zapa_page").hide();
            $("#address_page").show();
            if (this.model.get("client_address") !== null){
                $("#address_search").val(this.model.get("client_address").split(",")[0]);    
            }
            //$("#address_search").focus();
        }else if (state == STATE_CONFIRM_PICKUP){
            $(".zapa_page").hide();
            $("#address_confirm").val(this.model.get("client_address").split(",")[0]);
            $("#confirm_page").show();
            this.setNumberWidth();
            $("#id_phone_number").val(this.model.get("client_phone_number"));
            setTimeout(function(){$("#confirm_page").trigger("touchend" )}, 500);
        }else if (state == STATE_MONITORING_TRIP){
            google.maps.event.trigger(this.map, 'resize');
            $("#mapwrapper").css("top", "0");
            $(".zapa_page").hide();
            $("#map_page").show();
            $("#address_action").hide();
            $("#pickup_button_container").hide();
            $(".hamburger_menu").hide();
            $("#cancel_button").show();
            $(".pcontainer").show();
            this.map.setOptions({draggable: false});
        }else if (state == STATE_TRIP_DONE){
            $(".zapa_page").hide();
            $("#trip_summary_calldriver_link").attr("href", "tel://" + this.model.get("trip").get("driver_number"));
            $("#trip_summary").show();
            if (this.driverMarker !== null){
                this.driverMarker.setMap(null);
            }
            this.driverMarker = null;
            this.available_drivers = null;
        }else if (state == STATE_TRIP_FAILED){
        }
        return this;
    },
    appStateChanged: function(){
        this.render();
        var state = this.model.get("state");
        switch (state){
            case STATE_INITIALIZING:
                this.statusbar.text("Initializing...");
                break;
            case STATE_GETTING_LOCATION:
                this.statusbar.text("Getting your location...");
                break;
            case STATE_GETTING_ADDRESS:
                this.addressbox.val("Updating...");
                break;
            case STATE_READY:
                $("#statusbar_text").show();
                this.map.setOptions({
                    zoomControl: true
                });
                this.updateDriverStatusbar();
                this.addressbox.val(this.model.get("client_address"));
                break;
            case STATE_TRIP_FAILED:
                var failed_reason = this.model.get("trip").get("status");
                if (failed_reason == TRIP_STATUS_FAILED || failed_reason == TRIP_STATUS_NO_DRIVERS){
                    this.showPopup("No drivers", "Unfortunately no drivers responded. Please try again.", "Retry", "Cancel", this.newTrip, this.model.gotoReady);
                }else if (failed_reason == TRIP_STATUS_CANCELLED){
                    window.clearInterval(this.updateCountdownInterval);
                    this.showInfoPopup("Trip cancelled.", "Your trip has been cancelled.");
                    this.model.gotoReady();
                }else if (failed_reason == TRIP_STATUS_DRIVER_CANCELLED){
                    this.showPopup("Driver Cancelled", "Unfortunately your driver cancelled.", "Retry", "Cancel", this.newTrip, this.model.gotoReady);
                }else if (failed_reason == TRIP_STATUS_NO_SHOW){
                    this.showInfoPopup("Trip Cancelled", "Unfortunately your driver could not get hold of you.");
                    this.model.set("state", STATE_READY);
                }
                break;
            case STATE_TRIP_DONE:
                this.statusbar.text("Trip done");
                break;
            case STATE_GETTING_LOCATION_ERROR:
                this.showInfoPopup("We can't find your location", "Please turn on your location services under settings or enter your address manually");
                this.model.set("state", STATE_READY);
                break;

        }
    },
    tripStatusUpdated: function(){
        this.render();
        //this function is used to monitor the trip
        if (this.model.get("state") != STATE_MONITORING_TRIP){
            return;
        }
        var status = this.model.get("trip").get("status");
        switch (status){
            case TRIP_STATUS_INQUIRY:
                this.statusbar.text("Searching for cabs");
                $("#mapwrapper").css("top", "0")
                this.map.setOptions({
                    zoomControl: false
                });
                this.startTripCountdown();
                break;
            case TRIP_STATUS_ACCEPTED:
                window.clearInterval(this.updateCountdownInterval);
                this.statusbar.text("Your driver is on his way.");
                $("#over_map").hide();
                $("#statusbar_text").show();
                $("#trip_countdown_bar").css('width', '0%');
                $(".pcontainer").hide();
                this.updateClientMarker();
                this.updateDriverMarker();
                this.updateDriverDetails();
                $("#mapwrapper").css("top", "80px");
                break;
            case TRIP_STATUS_ONBOARD:
                this.statusbar.text("You are on board");
                this.hideClientMarker();
                $("#cancel_zap_container").hide();
                $("#over_map").hide();
                $("#statusbar_text").show();
                $("#trip_countdown_bar").css('width', '0%');
                $(".pcontainer").hide();
                $("#cancel_button").hide();
                this.updateDriverMarker();
                var trip = this.model.get("trip");
                $(".driver_name").text(trip.get("driver_name"));
                $(".driver_company").text("(" + trip.get("driver_company") + ")");
                $(".driver_license_plate").text(trip.get("driver_license_plate"));
                $(".driver_call_button").attr('href', 'tel:' + trip.get("driver_number"));
                $("#mapwrapper").css("top", "80px");
                break;



        }
    },
    updateDriverDetails: function(){
        var trip = this.model.get("trip");
        $(".driver_name").text(trip.get("driver_name"));
        if (trip.get("driver_company") !== null){
            $(".driver_company").text("(" + trip.get("driver_company") + ")");
        }else{
            $(".driver_company").text("");
        }
        $(".driver_company").text("(" + trip.get("driver_company") + ")");
        $(".driver_license_plate").text(trip.get("driver_license_plate"));
        $(".driver_call_button").attr('href', 'tel:' + trip.get("driver_number"));
    },
    newTrip: function(){
        $(".form-control").blur();
        this.model.get("trip").set("status", null);
        this.model.get("trip").set("uuid", null);
        if (validPhoneNumber(this.phone_number_input.val()) === true){
            $("#zap_button").attr("disabled", true);
            $("#zap_button").html("Zapping...");
            this.model.startNewTrip();
            setTimeout(function(){
                $("#zap_button").attr("disabled", false);
                $("#zap_button").html("Zap me a cab!");
                }, 5000); //debounce
        }else{
            this.showInfoPopup("Invalid phone number", "Please enter a valid phone number.")
        }
    },
    cancelTrip: function(){
        this.showPopup("Cancel Trip", "Are you sure you want to cancel this trip?", "Yes", "No", this.model.cancelTrip, function(){});
    },
    showPopup: function(title, message, primary_caption, secondary_caption, primary_callback, secondary_callback){
        bootbox.hideAll(); //is this ok?
        var buttons = {};
        if (primary_caption !== null){
            buttons.primary = {label: primary_caption,
                               className: "btn-primary",
                               callback: primary_callback
                               }
        }
        if (secondary_caption !== null){
            buttons.secondary = {label: secondary_caption,
                                className: "btn-default",
                                callback: secondary_callback
                                }
        }
        bootbox.dialog({
            title: title,
            message: message,
            buttons: buttons
        });
    },
    showInfoPopup: function(title, message){
        bootbox.hideAll(); //is this ok?
        bootbox.dialog({
            title: title,
            message: message,
            buttons: {
                primary: {
                    label: "Ok",
                    className: "btn-primary",
                    callback: function(){}
                }
            }
        });
    },
    goToCurrentLocation: function(){
        this.model.getLocation();
        return;
        
    },
    pickupLocationChanged:function(){
        //move to model
        var c_pos = this.model.get("pickup_location");
        this.setMapCenter(c_pos);
    },
    updateDriverMarker: function(){
        var tripobj = this.model.get("trip");
        if (this.driver_markers.length > 0){
            for (var x in this.driver_markers){
                this.driver_markers[x].setMap(null);
            }
            this.driver_markers = [];
        }

        var c_pos = tripobj.get("driver_position").split(";");
        var pos = new google.maps.LatLng(c_pos[0], c_pos[1]);
        if (this.driverMarker === null) {
            var marker = new google.maps.Marker({
                position: pos,
                map: this.map,
                icon: this.driverMarkerImage,
            });
            this.driverMarker = marker;
        }
        if (!this.driverMarker.map) {
           this.driverMarker.setMap(this.map);
        }
        this.driverMarker.setPosition(pos);

        var driver_distance = tripobj.get("driver_distance");
        var driver_position = tripobj.get("driver_position").split(";");
        var driver_position_latlng = new google.maps.LatLng(driver_position[0], driver_position[1]);

        var client_position = this.model.get("pickup_location");
        var client_position_latlng = new google.maps.LatLng(client_position[0], client_position[1]);

        if (this.map && tripobj.get("status") == TRIP_STATUS_ACCEPTED){
            if (this.map_bounds_set === 0 || 
                (driver_distance < 1000 && this.map_bounds_set === 1) || 
                !this.map.getBounds().contains(driver_position_latlng) || 
                (driver_distance <= 300 && this.map_bounds_set === 1) || 
                (driver_distance <= 300 && this.map_bounds_set === 2)){
                var bounds = new google.maps.LatLngBounds();
                bounds.extend(driver_position_latlng);
                bounds.extend(client_position_latlng);
                this.map.fitBounds(bounds);
                this.map_bounds_set += 1;
            }

            if (driver_distance <= 300){
                this.statusbar.text("Your driver has arrived");
                $("#cancel_button").hide();
            }

        }else if (this.map && tripobj.get("status") == TRIP_STATUS_ONBOARD){
            this.map.setZoom(TRACK_ZOOM);
            this.setMapCenter(driver_position);
        }

    },
    startTripCountdown: function(){
        this.render();
        $("#trip_countdown_bar").css('width', '0%');
        $(".pcontainer").show();

        var totalTime = 99;
        this.updateProgressbar(totalTime);

        this.updateCountdownInterval = setInterval(function(){
            totalTime -= 1;
            if(totalTime >= 0) {
                view.updateProgressbar(totalTime);
            }
            else {
                if (view.updateCountdownInterval) {
                    clearInterval(view.updateCountdownInterval);
                }
                view.updateCountdownInterval = null;
            }
        }, 1000);

    },
    updateProgressbar: function(count){
        $("#trip_countdown_bar").css('width', (100 - count)+'%');
    },
    eventManualAddressEnter: function(){
        this.model.set("state", STATE_MANUAL_ADDRESS_ENTER);
    },
    eventPickUp: function(){
        this.model.set("state", STATE_CONFIRM_PICKUP);
    },
    mapCenterChanged: function(){
        if (this.model.get("state") == STATE_READY){
            var lat_lng_position = this.map.getCenter();
            var new_c_pos = [parseFloat(lat_lng_position.lat()), parseFloat(lat_lng_position.lng())];
            this.model.set("pickup_location", new_c_pos);
        }
    },
    mapDragged: function(){
            this.addressbox.val("Updating...");
            $('#pickup_button').attr('disabled', true);
    },
    updateAvailableDrivers: function(){
        if (this.model.get("state") == STATE_READY){
            console.log("clearing driver markers to pin updated markers");
            for (var x in this.driver_markers){
                this.driver_markers[x].setMap(null);
            }
            this.driver_markers = [];
            var drivers = this.model.get("available_drivers");
            var icon = this.driverMarkerImage;
            for (var x in drivers){
                var location = drivers[x];
                var marker = new google.maps.Marker({
                    position: new google.maps.LatLng(location.latitude,location.longitude),
                    map: this.map,
                    icon: icon // This path is the custom pin to be shown. Remove this line and the proceeding comma to use default pin
                });
                marker.setMap(this.map);
                this.driver_markers.push(marker);
            }
            this.updateDriverStatusbar();
            
        }
    },
    updateDriverStatusbar: function(){
        var drivertext = "";
            if (this.model.get("available_drivers") === null){
                drivertext = "Searching for drivers...";
            }else if (this.model.get("available_drivers").length == 1){
                drivertext = "1 driver within range";
            }else if (this.model.get("available_drivers").length > 1){
                drivertext = this.model.get("available_drivers").length + " drivers within range";
            }else{
                drivertext = "Sorry, no cabs available in this area"
            }
            this.statusbar.text(drivertext);
    },
    setMapCenter: function(pos){
        this.map.setCenter(new google.maps.LatLng(pos[0], pos[1]));
        this.map_dragged = false;
         this.map_first_drag_listener = google.maps.event.addListener(this.map, 'drag', function() {
             this.map_dragged = true;
             google.maps.event.removeListener(this.map_first_drag_listener);
         });
    },
    hideClientMarker: function(){
        if (this.clientMarker !== null){
            this.clientMarker.setMap(null);
            this.clientMarker = null;
        }
    },
    updateClientMarker: function(){
        var c_pos = this.model.get("pickup_location");
        var pos = new google.maps.LatLng(c_pos[0], c_pos[1]);

        if (!this.clientMarker){
            var marker = new google.maps.Marker({
                position: pos,
                map: this.map,
                icon: this.clientMarkerImage,
            });
            this.clientMarker = marker;
        }
        if (!this.clientMarker.map) {
           this.clientMarker.setMap(this.map);
           this.map.setZoom(TRACK_ZOOM);
        }

        this.clientMarker.setPosition(pos);
    },

});


//var trip = new Trip();
var appmodel = new AppModel();
var view = new AppView({model: appmodel});
window.view = view;

});