var landing = new function Landing(map) {
    this.client = null;
    this.driverMarkers = [];
    this.initialLocation = null;

    this.driverUpdateInterval = null;

    /** update the map of drivers*/
    this.clearDriverLocations = function() {
        for (var i = 0; i < this.driverMarkers.length; i++) {
            this.driverMarkers[i].setMap(null);
        }

        this.driverMarkers = [];
    };

    this.updateDriverLocations = function(map, locations) {
        this.clearDriverLocations();

        for (var i = 0; i < locations.length; i++) {
            var location = locations[i];
            // Add Marker
            var icon = landing.availableMarker;

            if(location.status == "busy") {
                icon = landing.busyMarker;
            }

            if(location.status == "trip") {
                icon = landing.tripMarker;
            }

            var marker = new google.maps.Marker({
                position: new google.maps.LatLng(location.latitude,location.longitude),
                map: map,
                icon: landing.availableMarker // This path is the custom pin to be shown. Remove this line and the proceeding comma to use default pin
            });
            this.driverMarkers.push(marker);
        }
    };

    this.updateClientLocation = function(position, force, zoom) {
        var showClientMarker = true;

        if (position === null) {
            position = new google.maps.LatLng(-27.747455, 25.4236375);
            showClientMarker = false;
        }

        if (landing.currentUserPos &&
            (landing.currentUserPos.lat()).toFixed(6) == (position.lat()).toFixed(6) &&
            (landing.currentUserPos.lng()).toFixed(6) == (position.lng()).toFixed(6)) {
            // We don't update it if we haven't moved
            if (!force) {
                return;
            }
        }

        if (zoom === undefined) {
            if (landing.dynamicMap) {
                zoom = 14;
            }
            else {
                zoom = 10;
            }
        }

        landing.currentUserPos = position;

        if (landing.dynamicMap) {
            if (this.client === null) {
                if (showClientMarker) {
                    this.client = new google.maps.Marker({
                        position: position,
                        map: landing.map,
                        icon: landing.userMarker,
                        zIndex: google.maps.Marker.MAX_ZINDEX + 1
                    });
                }
            }
            else {
                this.client.setPosition(position);
            }

            landing.map.panTo(position);
            landing.map.setZoom(zoom);
            landing.fetchDriverLocations(position, zoom);
        }
        else {
            // Fetch drivers around this new location and change static map
            landing.fetchDriverLocations(position, zoom, function() {
                pos = {
                    "latitude": position.lat(),
                    "longitude": position.lng(),
                };

                var baseMap = landing.staticMapURL($(document).width(),
                    $(document).height() - 52, zoom, pos.latitude, pos.longitude,
                    pos, landing.currentDrivers);

                $(".driver-map-holder").css(
                  "background", "url("+baseMap+") no-repeat").css(
                  "background-size", "cover");
            });
        }
    };

    /** start zapping a cab */
    this.beginZap = function() {
        $(".search-step-1").hide();
        $(".search-step-2").show();

        history.pushState(
            {"state":"confirm"},
            "Zapacab - Confirm your location");

        var top = $(window).height() - 100;
        $(".map-actions").css({"top": top+"px"});
        $(".map-actions").animate({
            top: "50px", bottom: "0px"
        }, 500);

        $(".driver-map-holder").addClass("covered");

        return false;
    };

    this.cancelZap = function() {
        $(".map-actions").attr("style", "");
        $(".driver-map-holder").removeClass("covered");
        $(".search-step-2").hide();
        $(".search-step-1").show();
    };

    this.goToCurrentLocation = function() {
        if (currentGeoPosition) {
            this.updateClientLocation(currentGeoPosition, true);
            clientLocationManual = false;
            $("#location-entry").val(currentGeoAddress);
        }
    };

    this.fetchDriverLocations = function(position, zoom, callback) {
        console.log("Fetch driver locations around: " + position);

        if (this.driverUpdateInterval) {
            clearInterval(this.driverUpdateInterval);
            this.driverUpdateInterval = null;
        }

        this.driverUpdateInterval = setInterval(function() {
            landing.fetchDriverLocations(position, zoom, callback);
        }, 30000);

        if (zoom === undefined) {
            zoom = 10;
        }

        $.get("/api/latest/c/drivers/", {
                "latitude": position.lat(),
                "longitude": position.lng(),
                "zoom": zoom
            },
            function(data) {
                landing.updateDriverLocations(landing.map, data.drivers);
                landing.currentDrivers = data.drivers;

                if (callback) {
                    callback();
                }
            }
        );
    };

    /** show a static map with the users current location*/
    this.showStaticMap = function() {
        var baseMap = landing.staticMapURL(
            $(document).width(), $(document).height() - 52, 4,
            -27.747455, 25.4236375);

        $(".driver-map-holder").css(
            "background", "url("+baseMap+") no-repeat").css(
            "background-size", "cover");

        console.log("showStaticMap");
        if (clientLocation) {
            landing.updateClientLocation(clientLocation, true);
        }
        else {
            landing.updateClientLocation(landing.initialLocation, true, 4);
        }
    };

    this.staticMapURL = function(width, height, zoom, lat, lon, userPos, drivers) {
        var url = ("http://maps.googleapis.com/maps/api/staticmap?" +
            "center=" + lat + "," + lon + "&zoom=" + zoom + "&size=" + Math.ceil(width/1) + "x" + Math.ceil(height/1) +
            "&maptype=roadmap&sensor=true&visual_refresh=true&scale=1&format=jpg");

        if (drivers && drivers.length) {
            url += "&markers=scale:2|icon:http://www.zapacab.com/static/images/icon_taxi_available.png";
            for (var i = 0; i < drivers.length; i++) {
                var driver = drivers[i];
                url += "|" + driver.latitude + "," + driver.longitude;
            }
        }

        if (userPos) {
            url += "&markers=scale:2|icon:http://www.zapacab.com/static/images/icon_client.png";
            url += "|" + userPos.latitude + "," + userPos.longitude;
        }

        return url;
    };

    this.showDynamicMap = function() {
        var mapOptions = {
            center: new google.maps.LatLng(-27.747455, 25.4236375),
            zoom: 4,
            mapTypeId: google.maps.MapTypeId.ROADMAP,
            streetViewControl: false,
            mapTypeControl: false,
            zoomControl: false,
            styles: [
                {
                    featureType: "poi",
                    elementType: "labels",
                    stylers: [{
                        visibility: "off"
                    }]
                }
            ]
        };

        landing.map = new google.maps.Map(document.getElementById("map-canvas"),
            mapOptions);

        // Define Marker properties
        landing.availableMarker = new google.maps.MarkerImage("/static/images/icon_taxi_available.png",
            null, null, null, new google.maps.Size(32, 44));
        landing.busyMarker = new google.maps.MarkerImage("/static/images/icon_taxi_busy.png",
            null, null, null, new google.maps.Size(32, 44));
        landing.tripMarker = new google.maps.MarkerImage("/static/images/icon_taxi_passenger.png",
            null, null, null, new google.maps.Size(32, 44));
        landing.userMarker = new google.maps.MarkerImage('/static/images/icon_client.png',
            null, null, null,new google.maps.Size(32, 44));

        if (clientLocation) {
            landing.updateClientLocation(clientLocation, true);
        }
        else {
            landing.updateClientLocation(landing.initialLocation, true, 4);
        }
    };
};

$(document).ready(function() {
  if($(window).height() > 320){
    landing.dynamicMap = true;
    landing.showDynamicMap();
  }else{
    landing.dynamicMap = false;
    landing.showStaticMap();
  }

  // Revert to a previously saved state
  window.addEventListener('popstate', function(event) {
    landing.cancelZap();
  });
});
