var tripSearch = new function Search() {
    this.confirm = function(address, passengers) {
        geocoder = new google.maps.Geocoder();
        geocoder.geocode({
            "language": "en",
            "address": address
        }, function(results, status) {
            gmapsResults = results;

            var address = getPrettyAddress(results[0]);
            var area = getSuburb(results[0]);
            var city = getCity(results[0]);

            var url = "/c/simple-confirmed?" + $.param({
                "address": address,
                "area": area,
                "city": city,
                "latitude": gmapsResults[0].geometry.location.lat(),
                "longitude": gmapsResults[0].geometry.location.lng(),
                "passengers": passengers
            });

            window.location = url;
        });
    };
};

var trip = new function Trip() {
    this.tripId = null;
    this.location = null;
    this.map = null;
    this.driverPosition = null;

    this.driverMarker = null;
    this.clientMarker = null;

    this.updateInterval = 1000;

    this.tripStatus = null;

    this.init = function(tripId) {
        this.tripId = tripId;
    };

    this.fail = function() {
        $(".zc-dialog").hide();
        $("#failed-dialog").show();
        trip.updateInterval = 100000;

        if (trip.updateTimeout) {
            clearTimeout(trip.updateTimeout);
            trip.updateTimeout = null;
        }
    };

    this.redirect = function() {
        if (trip.location) {
            location.href="/location/" + trip.location;
        }
        else {
            location.href='/';
        }
    };

    this.updateTripStatus = function() {
        $.ajax({
            type: "GET",
            url: "/api/latest/c/update/" + this.tripId + "/",
            cache: false,
            dataType: "json",
            success: function(data) {
                var t = data.trip;

                if (t.status == 'accepted') {
                    if (!trip.driverPosition) {
                        $(".zc-dialog").hide();
                        $("#arriving-dialog").show();
                    }

                    if (t.support_num) {
                        $(".support_num").html("Call support " + t.support_num);
                        $(".support_num").attr('href','tel:'+ t.support_num);
                    }
                    else {
                        $(".support_num").html("Call support 0836334370");
                    }

                    if (t.driver_number) {
                        if (t.driver_company) {
                            $("#cab-info").html(t.driver_company + " driver " + t.driver_name + " is on the way<br />Call: " +
                                "<a href='tel:" + t.driver_number + "' class='ui-link'>" + t.driver_number + "</a>");
                        }
                        else {
                            $("#cab-info").html(t.driver_name + " is on the way<br />Call: " +
                                "<a href='tel:" + t.driver_number + "' class='ui-link'>" + t.driver_number + "</a>");
                        }
                    }

                    $("#arriving-time").html(
                        parseInt(t.driver_distance / 500 + 1, 10));
                }
                else if (t.status == 'onboard') {
                    $(".zc-dialog").hide();
                    $("#transit-dialog").show();
                    trip.updateInterval = 10000;
                }
                else if (t.status == 'failed' || t.status == 'no drivers') {
                    if (t.status != trip.tripStatus) {
                        trip.fail();
                    }
                }
                else if (t.status == 'cancelled') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert('Your trip has been cancelled.', function() {
                            trip.redirect();
                        });
                    }
                }
                else if (t.status == 'driver cancelled') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert("Unfortunately your driver has cancelled. Please try again if you still need a cab", function() {
                            trip.redirect();
                        });
                    }
                }
                else if (t.status == 'no show') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert("Unfortunately your driver couldn't contact you and has cancelled. Please try again if you still need a cab", function() {
                            trip.redirect();
                        });
                    }
                }
                else if (t.status == 'finished') {
                    if (t.status != trip.tripStatus) {
                        bootbox.alert('Your trip has been finished. ' +
                            'We hope you had a great ride and see you again soon.', function() {
                                trip.redirect();
                        });
                    }
                }

                trip.tripStatus = t.status;

            }
        });

        this.updateTimeout = setTimeout(function() {
            trip.updateTripStatus();
        }, trip.updateInterval);
    };

    this.cancelTrip = function() {
        bootbox.confirm('Are you sure you want to cancel this trip?', function(result) {
            if (result) {
                $.ajax({
                  type: 'GET',
                  url: "/api/latest/c/cancel/" + trip.tripId + "/",
                  cache: false,
                  success: function(data) {
                    trip.tripStatus = "cancelled";
                    bootbox.alert('Your trip has been cancelled.', function() {
                        trip.redirect();
                    });
                  }
                });
            }
        });
    };

    this.startCountdownTimer = function(time, selector){
        // A countdown timer while searching for trips
        var div = $(selector);
        div.show();

        var totalTime = time;
        trip.updateCountdownTimer(totalTime, selector);

        trip.updateCountdownInterval = setInterval(function(){
            totalTime -= 1;
            if(totalTime >= 0) {
                trip.updateCountdownTimer(totalTime, selector);
            }
            else {
                if (trip.updateCountdownInterval) {
                    clearInterval(trip.updateCountdownInterval);
                }

                trip.updateCountdownInterval = null;
                if (!trip.tripStatus || trip.tripStatus == "inquiry") {
                    trip.fail();
                }
            }
        }, 1000);
    };

    this.updateCountdownTimer = function(time, selector) {
        $(selector).find(".time-left").html(time);
    };
};
